@@ -9,6 +9,8 @@ from steam_sdk.data.DataSettings import DataSettingsSTEAM
 from steam_sdk.builders.BuilderModel import BuilderModel
 from steam_sdk.data.DataAnalysis import DataAnalysis, AnalysisStep
 from steam_sdk.drivers.DriverLEDET import DriverLEDET
+from steam_sdk.drivers.DriverPSPICE import DriverPSPICE
+from steam_sdk.drivers.DriverPyBBQ import DriverPyBBQ
 from steam_sdk.utils.make_folder_if_not_existing import make_folder_if_not_existing
 from steam_sdk.utils.rgetattr import rgetattr
 from steam_sdk.utils.sgetattr import rsetattr
@@ -186,10 +188,15 @@ class AnalysisSTEAM:
                 step = def_step.step
                 print('{}'.format(step.step_name))
 
+        # Print analysis sequence
         if verbose: print('Defined sequence of analysis steps:')
         for s, seq_step in enumerate(self.data_analysis.AnalysisStepSequence):
             if verbose: print('Step {}/{}: {}'.format(s+1, len(self.data_analysis.AnalysisStepSequence), seq_step))
 
+        # Run analysis (and re-print analysis steps)
+        if verbose: print('Analysis started.')
+        for s, seq_step in enumerate(self.data_analysis.AnalysisStepSequence):
+            if verbose: print('Step {}/{}: {}'.format(s + 1, len(self.data_analysis.AnalysisStepSequence), seq_step))
             current_step = (i for i, v in enumerate(step_definitions) if step_definitions[i].step.step_name == seq_step)
             current_step = next(current_step, None)
             # if verbose: print('Index of current step: {}'.format(current_step))
@@ -222,17 +229,25 @@ class AnalysisSTEAM:
         flag_build      = step.flag_build
         flag_dump_all   = step.flag_dump_all
         flag_plot_all   = step.flag_plot_all
+        flag_json       = step.flag_json
         output_folder   = self.output_path
         relative_path_settings = ''
 
         # Build the model
         BM = BuilderModel(file_model_data=file_model_data, case_model=case_model, software=software,
-                          verbose=verbose_of_step, flag_build=flag_build, flag_dump_all=flag_dump_all, flag_plot_all=flag_plot_all,
+                          verbose=verbose_of_step, flag_build=flag_build,
+                          flag_dump_all=flag_dump_all, flag_plot_all=flag_plot_all, flag_json=flag_json,
                           output_path=output_folder, relative_path_settings=relative_path_settings)
 
         # Build simulation file
         if step.simulation_number:
-            self.setup_sim_LEDET(simulation_name=step.simulation_name, sim_number=step.simulation_number)
+            if 'LEDET' in step.software:
+                flag_yaml = True  # Hard-coded for the moment
+                self.setup_sim_LEDET(simulation_name=step.simulation_name, sim_number=step.simulation_number, flag_yaml=flag_yaml, flag_json=flag_json)
+            if 'PyBBQ' in step.software:
+                self.setup_sim_PyBBQ(simulation_name=step.simulation_name, sim_number=step.simulation_number)
+            if 'PSPICE' in step.software:
+                self.setup_sim_PSPICE(simulation_name=step.simulation_name, sim_number=step.simulation_number)
 
         # Add the reference to the model in the dictionary
         self.list_models[step.model_name] = BM
@@ -256,6 +271,7 @@ class AnalysisSTEAM:
         # Change the value of the selected variable
         for v, value in enumerate(step.variable_value):
             BM = self.list_models[step.model_name]  # original BuilderModel object
+            case_model = BM.case_model  # model case (magnet, conductor, circuit)
 
             if 'Conductors[' in step.variable_to_change:  # Special case when the variable to change is the Conductors key
                 if verbose:
@@ -263,7 +279,7 @@ class AnalysisSTEAM:
                     conductor_variable_to_change = step.variable_to_change.split('].')[1]
                     print('Variable {} is treated as a Conductors key. Conductor index: #{}. Conductor variable to change: {}.'.format(step.variable_to_change, idx_conductor, conductor_variable_to_change))
 
-                    old_value = rgetattr(BM.model_data.Conductors[idx_conductor], conductor_variable_to_change)
+                    old_value = self.get_attribute_model(case_model, BM, conductor_variable_to_change, idx_conductor)
                     print('Variable {} changed from {} to {}.'.format(conductor_variable_to_change, old_value, value))
 
                 if len_new_model_name > 0:  # Make a new copy of the BuilderModel object, and change it
@@ -278,25 +294,89 @@ class AnalysisSTEAM:
 
             else:  # Standard case when the variable to change is not the Conductors key
                 if verbose:
-                    old_value = rgetattr(BM.model_data, step.variable_to_change)
+                    old_value = self.get_attribute_model(case_model, BM, step.variable_to_change)
                     print('Variable {} changed from {} to {}.'.format(step.variable_to_change, old_value, value))
 
                 if len_new_model_name > 0:  # Make a new copy of the BuilderModel object, and change it
                     self.list_models[step.new_model_name[v]] = deepcopy(BM)
                     BM = self.list_models[step.new_model_name[v]]
-                    rsetattr(BM.model_data, step.variable_to_change, value)
+                    self.set_attribute_model(case_model, BM, step.variable_to_change, value)
                     if verbose:
                         print('Model {} copied to model {}.'.format(step.model_name, step.new_model_name[v]))
 
                 else:  # Change the original BuilderModel object
-                    rsetattr(BM.model_data, step.variable_to_change, value)
+                    self.set_attribute_model(case_model, BM, step.variable_to_change, value)
 
             # Build simulation file
             if len_simulation_numbers > 0:
                 simulation_number = step.simulation_numbers[v]
                 if 'LEDET' in step.software:
-                    BM.buildLEDET()
-                    self.setup_sim_LEDET(simulation_name=step.simulation_name, sim_number=simulation_number)
+                    flag_json = BM.flag_json
+                    BM.buildLEDET(case_model=case_model)
+                    self.setup_sim_LEDET(simulation_name=step.simulation_name, sim_number=simulation_number, flag_json=flag_json)
+                if 'PyBBQ' in step.software:
+                    BM.buildPyBBQ()
+                    self.setup_sim_PyBBQ(simulation_name=step.simulation_name, sim_number=simulation_number)
+                if 'PSPICE' in step.software:
+                    BM.buildPSPICE()
+                    self.setup_sim_PSPICE(simulation_name=step.simulation_name, sim_number=simulation_number)
+
+    def get_attribute_model(self, case_model: str, builder_model: BuilderModel, name_variable: str, idx_conductor: int = None):
+        '''
+        Helper function used to get an attribute from a key of the model data.
+        Depending on the model type (circuit, magnet, conductor), the data structure to access is different.
+        Also, there is a special case when the variable to read is a sub-key of the Conductors key. In such a case, an additional parameter idx_conductor must be defined (see below).
+        :param case_model: Model type
+        :param builder_model: BuilderModel object to access
+        :param name_variable: Name of the variable to read
+        :param idx_conductor: When defined, a sub-key form the Conductors key is read. The index of the conductor to read is defined by idx_conductor
+        :return: Value of the variable to get
+        '''
+
+        if case_model == 'magnet':
+            if idx_conductor is None:  # Standard case when the variable to change is not the Conductors key
+                value = rgetattr(builder_model.model_data, name_variable)
+            else:
+                value = rgetattr(builder_model.model_data.Conductors[idx_conductor], name_variable)
+        elif case_model == 'conductor':
+            if idx_conductor is None:  # Standard case when the variable to change is not the Conductors key
+                value = rgetattr(builder_model.conductor_data, name_variable)
+            else:
+                value = rgetattr(builder_model.conductor_data.Conductors[idx_conductor], name_variable)
+        elif case_model == 'circuit':
+            value = rgetattr(builder_model.circuit_data, name_variable)
+        else:
+            raise Exception(f'Model type not supported: case_model={case_model}')
+        return value
+
+    def set_attribute_model(self, case_model: str, builder_model: BuilderModel, name_variable: str, value_variable: float, idx_conductor: int = None):
+        '''
+        Helper function used to set a key of the model data to a certaub value.
+        Depending on the model type (circuit, magnet, conductor), the data structure to access is different.
+        Also, there is a special case when the variable to change is a sub-key of the Conductors key. In such a case, an additional parameter idx_conductor must be defined (see below).
+        :param case_model: Model type
+        :param builder_model: BuilderModel object to access
+        :param name_variable: Name of the variable to change
+        :param value_variable: New value of the variable of the variable
+        :param idx_conductor: When defined, a sub-key form the Conductors key is read. The index of the conductor to read is defined by idx_conductor
+        :return: Value of the variable to get
+        '''
+
+        if case_model == 'magnet':
+            if idx_conductor is None:  # Standard case when the variable to change is not the Conductors key
+                rsetattr(builder_model.model_data, name_variable, value_variable)
+            else:
+                rsetattr(builder_model.model_data.Conductors[idx_conductor], name_variable, value_variable)
+        elif case_model == 'conductor':
+            if idx_conductor is None:  # Standard case when the variable to change is not the Conductors key
+                rsetattr(builder_model.conductor_data, name_variable, value_variable)
+            else:
+                rsetattr(builder_model.conductor_data.Conductors[idx_conductor], name_variable, value_variable)
+        elif case_model == 'circuit':
+            rsetattr(builder_model.circuit_data, name_variable, value_variable)
+        else:
+            raise Exception(f'Model type not supported: case_model={case_model}')
+
 
     def step_modify_model_multiple_variables(self, step, verbose: bool = False):
         if verbose:
@@ -329,14 +409,16 @@ class AnalysisSTEAM:
         if verbose:
             print('All variables of step {} were changed.'.format(step.step_name))
 
+
     def step_run_simulation(self, step, verbose: bool = False):
         software = step.software
         simulation_name = step.simulation_name
+        simFileType = step.simFileType
         for sim_number in step.simulation_numbers:
             if verbose:
                 print('Running simulation of model {} #{} using {}.'.format(simulation_name, sim_number, software))
             # Run simulation
-            self.run_sim(software, simulation_name, sim_number, verbose)
+            self.run_sim(software, simulation_name, sim_number, simFileType, verbose)
 
 
     def step_postprocess(self, step, verbose: bool = False):
@@ -384,14 +466,38 @@ class AnalysisSTEAM:
                     shutil.copyfile(file_to_copy, file_copied)
                     print('Field map file {} copied to {}.'.format(file_to_copy, file_copied))
 
+            elif 'PSPICE' in software:
+                local_PSPICE_folder = Path(self.settings.local_PSPICE_folder)
+                local_model_folder = Path(local_PSPICE_folder / simulation_name).resolve()
+                # Make magnet input folder
+                make_folder_if_not_existing(local_model_folder, verbose=verbose)
+
+                # Copy lib files from the output folder
+                list_lib_files = [entry for entry in os.listdir(self.output_path) if (simulation_name in entry) and ('.lib' in entry)]
+                for lib_file in list_lib_files:
+                    file_to_copy = os.path.join(self.output_path, lib_file)
+                    file_copied = os.path.join(local_model_folder, lib_file)
+                    shutil.copyfile(file_to_copy, file_copied)
+                    print('Lib file {} copied to {}.'.format(file_to_copy, file_copied))
+
+                # Copy stl files from the output folder
+                list_stl_files = [entry for entry in os.listdir(self.output_path) if (simulation_name in entry) and ('.stl' in entry)]
+                for stl_file in list_stl_files:
+                    file_to_copy = os.path.join(self.output_path,   stl_file)
+                    file_copied  = os.path.join(local_model_folder, stl_file)
+                    shutil.copyfile(file_to_copy, file_copied)
+                    print('Stl file {} copied to {}.'.format(file_to_copy, file_copied))
+
             else:
-                raise Exception('Software not supported for automated folder setup.')
+                raise Exception(f'Software {software} not supported for automated folder setup.')
 
 
-    def setup_sim_LEDET(self, simulation_name, sim_number):
+    def setup_sim_LEDET(self, simulation_name, sim_number, flag_yaml=True, flag_json=False):
         '''
         Set up a LEDET simulation by copying the last file generated by BuilderModel to the output folder and to the
         local LEDET working folder. The original file is then deleted.
+        If flag_yaml=True, the model is set up to be run using a yaml input file.
+        If flag_json=True, the model is set up to be run using a json input file.
         '''
 
         # Unpack
@@ -401,27 +507,121 @@ class AnalysisSTEAM:
         # Add simulation number to the list
         self.list_sims.append(sim_number)
 
-        file_name_temp   = os.path.join(output_folder, simulation_name + '.xlsx')
-        file_name_output = os.path.join(output_folder, simulation_name + '_' + str(sim_number) + '.xlsx')
-        file_name_local  = os.path.join(local_LEDET_folder, simulation_name, 'Input', simulation_name + '_' + str(sim_number) + '.xlsx')
+        # Copy simulation file
+        list_suffix = ['.xlsx']
+        if flag_yaml == True:
+            list_suffix.append('.yaml')
+        if flag_json == True:
+            list_suffix.append('.json')
 
-        shutil.copyfile(file_name_temp, file_name_output)
-        print('Simulation file {} generated.'.format(file_name_output))
+        for suffix in list_suffix:
+            file_name_temp   = os.path.join(output_folder, simulation_name + suffix)
+            file_name_output = os.path.join(output_folder, simulation_name + '_' + str(sim_number) + suffix)
+            file_name_local  = os.path.join(local_LEDET_folder, simulation_name, 'Input', simulation_name + '_' + str(sim_number) + suffix)
+
+            shutil.copyfile(file_name_temp, file_name_output)
+            print('Simulation file {} generated.'.format(file_name_output))
+
+            shutil.copyfile(file_name_temp, file_name_local)
+            print('Simulation file {} copied.'.format(file_name_local))
+
+            os.remove(file_name_temp)
+            print('Temporary file {} deleted.'.format(file_name_temp))
+
+
+    def setup_sim_PSPICE(self, simulation_name, sim_number):
+        '''
+        Set up a PSPICE simulation by copying the last file generated by BuilderModel to the output folder and to the
+        local PSPICE working folder.
+        The simulation netlist and auxiliary files are copied in a new numbered subfoldered.
+        The original file is then deleted.
+        '''
+
+        # Unpack
+        output_folder = self.output_path
+        local_PSPICE_folder = Path(self.settings.local_PSPICE_folder)
+
+        # Add simulation number to the list
+        self.list_sims.append(sim_number)
+
+        # Make simulation folder
+        local_model_folder = os.path.join(local_PSPICE_folder, simulation_name, str(sim_number))
+        make_folder_if_not_existing(local_model_folder)
+
+        # Copy simulation file
+        file_name_temp = os.path.join(output_folder, simulation_name + '.cir')
+        file_name_local = os.path.join(local_model_folder, simulation_name + '.cir')
+        print('Simulation file {} generated.'.format(file_name_temp))
 
         shutil.copyfile(file_name_temp, file_name_local)
-        print('Simulation file {} generated.'.format(file_name_local))
+        print('Simulation file {} copied.'.format(file_name_local))
+
+        os.remove(file_name_temp)
+        print('Temporary file {} deleted.'.format(file_name_temp))
+
+        # Copy lib files from the output folder
+        list_lib_files = [entry for entry in os.listdir(output_folder) if (simulation_name in entry) and ('.lib' in entry)]
+        for lib_file in list_lib_files:
+            file_to_copy = os.path.join(output_folder, lib_file)
+            file_copied = os.path.join(local_model_folder, lib_file)
+            shutil.copyfile(file_to_copy, file_copied)
+            print('Lib file {} copied to {}.'.format(file_to_copy, file_copied))
+
+        # Copy stl files from the output folder
+        list_stl_files = [entry for entry in os.listdir(output_folder) if (simulation_name in entry) and ('.stl' in entry)]
+        for stl_file in list_stl_files:
+            file_to_copy = os.path.join(output_folder,   stl_file)
+            file_copied  = os.path.join(local_model_folder, stl_file)
+            shutil.copyfile(file_to_copy, file_copied)
+            print('Stl file {} copied to {}.'.format(file_to_copy, file_copied))
+
+
+    def setup_sim_PyBBQ(self, simulation_name, sim_number):
+        '''
+        Set up a PyBBQ simulation by copying the last file generated by BuilderModel to the output folder and to the
+        local PyBBQ working folder.
+        The original file is then deleted.
+        '''
+
+        # Unpack
+        output_folder = self.output_path
+        local_PyBBQ_folder = Path(self.settings.local_PyBBQ_folder)
+
+        # Add simulation number to the list
+        self.list_sims.append(sim_number)
+
+        # Make simulation folder
+        local_model_folder = os.path.join(local_PyBBQ_folder, simulation_name, str(sim_number))
+        make_folder_if_not_existing(local_model_folder)
+
+        # Copy simulation file
+        file_name_temp = os.path.join(output_folder, simulation_name + '.yaml')
+        file_name_local = os.path.join(local_model_folder, simulation_name + '.yaml')
+        print('Simulation file {} generated.'.format(file_name_temp))
+
+        shutil.copyfile(file_name_temp, file_name_local)
+        print('Simulation file {} copied.'.format(file_name_local))
 
         os.remove(file_name_temp)
         print('Temporary file {} deleted.'.format(file_name_temp))
 
 
-    def run_sim(self, software: str, simulation_name: str, sim_number: int, verbose: bool = False):
+    def run_sim(self, software: str, simulation_name: str, sim_number: int, simFileType: str = None, verbose: bool = False):
         '''
         Run selected simulation.
         The function applies a different logic for each simulation software.
         '''
         if software == 'LEDET':
             dLEDET = DriverLEDET(path_exe=self.settings.LEDET_path, path_folder_LEDET=self.settings.local_LEDET_folder, verbose=verbose)
-            dLEDET.run_LEDET(simulation_name, str(sim_number))
+            dLEDET.run_LEDET(simulation_name, str(sim_number), simFileType=simFileType)
+        elif software == 'PyBBQ':
+            local_model_folder_input  = os.path.join(self.settings.local_PyBBQ_folder, simulation_name, str(sim_number))
+            relative_folder_output = os.path.join(simulation_name, str(sim_number))
+            dPyBBQ = DriverPyBBQ(path_exe=self.settings.PyBBQ_path, path_folder_PyBBQ=self.settings.local_PyBBQ_folder, path_folder_PyBBQ_input=local_model_folder_input, verbose=verbose)
+            dPyBBQ.run_PyBBQ(simulation_name, outputDirectory=relative_folder_output)
+        elif software == 'PSPICE':
+            local_model_folder = Path(Path(self.settings.local_PSPICE_folder) / simulation_name / str(sim_number)).resolve()
+            dPSPICE = DriverPSPICE(path_exe=self.settings.PSPICE_path, path_folder_PSPICE=local_model_folder, verbose=verbose)
+            dPSPICE.run_PSPICE(simulation_name, suffix='')
         else:
-            raise Exception('Software not supported for automated running.')
+            raise Exception(f'Software {software} not supported for automated running.')
