@@ -32,12 +32,15 @@ class PermanentSettings(BaseModel):
     JAVA_jdk_path:       str = None  # full path to folder with java jdk
     CFunLibPath:         str = None  # path to dll files with material properties
     LEDET_path:          str = None  #
+    PyBBQ_path:          str = None  #
     ProteCCT_path:       str = None  #
     PSPICE_path:         str = None  #
     COSIM_path:          str = None  #
     FiQuS_path:          str = None  #
     PSPICE_library_path: str = None  #
     local_LEDET_folder:  str = None  # full path to local LEDET folder
+    local_PyBBQ_folder:  str = None  # full path to local PyBBQ folder
+    local_PSPICE_folder: str = None  # full path to local PSPICE folder
 
 ############################
 # Working folders
@@ -67,6 +70,7 @@ class MakeModel(BaseModel):
     flag_dump_all:     bool = None
     verbose:           bool = None
     flag_plot_all:     bool = None
+    flag_json:         bool = None
 
 class ModifyModel(BaseModel):
     """
@@ -111,6 +115,7 @@ class RunSimulation(BaseModel):
     software:           str = None
     simulation_name:    str = None
     simulation_numbers: List[int] = []
+    simFileType:        str = None
 
 class PostProcess(BaseModel):
     """
