@@ -37,6 +37,7 @@ class ParserPSPICE:
 
         # Initialize
         self.circuit_data = DataModelCircuit()
+        self.circuit_data.InitialConditions.initial_conditions = {}
         # Set flags indicating that the last read line corresponds to an item that might span to the next line
         self._set_all_flags_to_false()
 
@@ -90,6 +91,16 @@ class ParserPSPICE:
                     self.circuit_data.GlobalParameters.global_parameters[str(name)] = str(value)
                     continue
 
+                # Read initial conditions
+                # TODO: Known issue: this logic will not work if multiple initial conditions are defined in the same line
+                if '.IC ' in line:
+                    line_split = line.rstrip('\n').split('.IC ')
+                    line_split = line_split[1].strip(' ').split(' ')  # remove whitespaces at the beginning and end of the string, then divide it at the internal whitespace
+                    name  = line_split[0]
+                    value = line_split[1].strip('{').strip('}')
+                    self.circuit_data.InitialConditions.initial_conditions[str(name)] = str(value)
+                    continue
+
                 # Read options
                 if '.OPTION' in line:  # it also covers the case where ".OPTIONS" is written
                     self.flag_read_options = True
@@ -189,6 +200,21 @@ class ParserPSPICE:
                     self.circuit_data.Netlist[-1].value = str(value)
                     continue
 
+                # Special case: Pulsed power source
+                if 'PULSE' in line.replace(' ', ''):
+                    # Note: The following logic works for three different syntaxes: "PULSE(value)", "PULSE (value)" and "PULSE value"
+                    line_before, _, line_after = line.rstrip('\n').partition('(')  # Take the part of the string before and after the first "(" char
+                    name = line_before.strip(' ')
+                    line_before, _, line_after = line_after.rstrip(' ').partition(')')  # Take the part of the string before and after the first ")" char
+                    nodes = line_before.split(' ')
+                    line_after = line_after.partition('PULSE')[2]  # Take the part of the string after the first "VALUE " chars
+                    value = line_after.strip(' ( ) ')  # Strip spaces and bracket at the extremities
+                    self.circuit_data.Netlist[-1].type  = 'pulsed-source component'
+                    self.circuit_data.Netlist[-1].name  = str(name)
+                    self.circuit_data.Netlist[-1].nodes = nodes
+                    self.circuit_data.Netlist[-1].value = str(value)
+                    continue
+
                 # Special case: Controlled-source component
                 # E: Voltage-controlled voltage source
                 # F: Current-controlled current source
@@ -318,13 +344,17 @@ class ParserPSPICE:
 
         # Prepare stimuli
         rows_stimuli = []
-        for s in self.circuit_data.Stimuli.stimulus_files:
-            rows_stimuli.append(add_stimulus(s))
+        rows_stimuli.append(add_comment('**** Stimulus files ****'))  # Add header of this section
+        if self.circuit_data.Stimuli.stimulus_files:
+            for s in self.circuit_data.Stimuli.stimulus_files:
+                rows_stimuli.append(add_stimulus(s))
 
         # Prepare libraries
         rows_libraries = []
-        for s in self.circuit_data.Libraries.component_libraries:
-            rows_libraries.append(add_library(s))
+        if self.circuit_data.Libraries.component_libraries:
+            rows_libraries.append(add_comment('**** Component libraries ****'))  # Add header of this section
+            for s in self.circuit_data.Libraries.component_libraries:
+                rows_libraries.append(add_library(s))
 
         # Prepare global parameters
         rows_global_parameters = []
@@ -336,6 +366,15 @@ class ParserPSPICE:
             for name, value in self.circuit_data.GlobalParameters.global_parameters.items():
                 rows_global_parameters.append(add_global_parameter(name, value))
 
+        # Prepare initial conditions
+        rows_initial_conditions = []
+        if self.circuit_data.InitialConditions.initial_conditions:
+            # Add comment and .PARAM command before the first entry
+            rows_initial_conditions.append(add_comment('**** Initial conditions ****'))  # Add header of this section
+            # Add initial condition entries
+            for name, value in self.circuit_data.InitialConditions.initial_conditions.items():
+                rows_initial_conditions.append(add_initial_condition(name, value))
+
         # Prepare netlist
         rows_netlist = []
         # Check inputs
@@ -345,9 +384,10 @@ class ParserPSPICE:
                             '- standard component\n'
                             '- stimulus-controlled component\n'
                             '- controlled-source component\n'
+                            '- pulsed-source component\n'
                             '- parametrized component\n'
                             'Netlist cannot be generated.')
-        rows_global_parameters.append(add_comment('**** Netlist ****'))  # Add header of this section
+        rows_netlist.append(add_comment('**** Netlist ****'))  # Add header of this section
         for s, component in enumerate(self.circuit_data.Netlist):
             # Read keys
             name, nodes, value, parameters, type = \
@@ -367,6 +407,11 @@ class ParserPSPICE:
                     raise Exception('Netlist component in position #{} is of type {} and requires name, nodes, and value.'.format(s+1, type))
                 if verbose: print('Netlist entry {} in position #{} is treated as a stimulus-controlled component.'.format(name, s + 1))
                 rows_netlist.append(add_stimulus_controlled_component(name, nodes, value))
+            elif type == 'pulsed-source component':
+                if name == None or nodes == None or value == None:
+                    raise Exception('Netlist component in position #{} is of type {} and requires name, nodes, and value.'.format(s+1, type))
+                if verbose: print('Netlist entry {} in position #{} is treated as a pulsed-source component.'.format(name, s + 1))
+                rows_netlist.append(add_pulsed_source_component(name, nodes, value))
             elif type == 'controlled-source component':
                 if name == None or nodes == None or value == None:
                     raise Exception('Netlist component in position #{} is of type {} and requires name, nodes, and value.'.format(s+1, type))
@@ -460,6 +505,7 @@ class ParserPSPICE:
             rows_stimuli + \
             rows_libraries + \
             rows_global_parameters + \
+            rows_initial_conditions + \
             rows_netlist + \
             rows_options + \
             rows_analysis + \
@@ -559,6 +605,11 @@ def add_global_parameter(name: str, value: str):
     formatted_text = '+ ' + name + '={' + value + '}'
     return formatted_text
 
+def add_initial_condition(name: str, value: str):
+    ''' Format initial condition row '''
+    formatted_text = '.IC ' + name + ' {' + value + '}'
+    return formatted_text
+
 def add_standard_component(name: str, nodes: list, value: str):
     ''' Format standard component netlist row '''
     str_nodes = " ".join(nodes)  # string with space-separated nodes
@@ -572,6 +623,13 @@ def add_stimulus_controlled_component(name: str, nodes: list, value: str):
     formatted_text = name + ' (' + str_nodes + ') ' + str_stimulus
     return formatted_text
 
+def add_pulsed_source_component(name: str, nodes: list, value: str):
+    ''' Format pulsed-source component netlist row '''
+    str_nodes = " ".join(nodes)  # string with space-separated nodes
+    str_pulse = 'PULSE(' + value + ')'
+    formatted_text = name + ' (' + str_nodes + ') ' + str_pulse
+    return formatted_text
+
 def add_controlled_source_component(name: str, nodes: list, value: str):
     ''' Format controlled-source component netlist row '''
     str_nodes = " ".join(nodes)  # string with space-separated nodes
@@ -646,11 +704,6 @@ def add_probe(probe_type: str, probe_variables: list):
 
     for var in probe_variables:
         formatted_text = formatted_text + '\n' + '+ ' + var
-
-    # Make sure the maximum number of characters in each row does not exceed 132, which is the maximum that PSPICE supports
-    N_MAX_CHARS_PER_ROW = 130
-    formatted_text = textwrap.fill(formatted_text, N_MAX_CHARS_PER_ROW)
-    formatted_text = formatted_text.replace('\n', '\n+ ')  # Add "+ " at the beginning of each new line
     return formatted_text
 
 def add_auxiliary_file(file_to_add: str):
